{
  description = "A very basic flake";

  inputs = {
    nixpkgs.url = "github:nixos/nixpkgs?ref=nixos-unstable";
  };

  outputs = { self, nixpkgs }:
    let
      systems = [ "x86_64-linux" "aarch64-linux" "x86_64-darwin" "aarch64-darwin" ];
      shellForSystem = system:
        let
          pkgs = nixpkgs.legacyPackages.${system};
          clangVersion = builtins.elemAt
            (nixpkgs.lib.strings.splitString "." (nixpkgs.lib.strings.getVersion pkgs.libclang)) 0;
        in
        {
          formatter.${system} = nixpkgs.legacyPackages.${system}.nixpkgs-fmt;
          devShells.${system}.default = pkgs.mkShell {
            buildInputs = [
              pkgs.cargo
              pkgs.clippy
              pkgs.cmake
              pkgs.rust-analyzer
              pkgs.rustfmt
              pkgs.z3
              pkgs.zip
            ];
            LIBCLANG_PATH = "${pkgs.libclang.lib}/lib";
            BINDGEN_EXTRA_CLANG_ARGS = ''
              -I ${pkgs.z3.dev}/include
              -I ${pkgs.glibc.dev}/include
              -I ${pkgs.libclang.lib}/lib/clang/${clangVersion}/include
            '';
          };
        };
    in
    builtins.foldl' (acc: sys: (shellForSystem sys) // acc) { } systems;
}
